#!/bin/perl -w
use strict;
use Class::Struct;
use Storable;
use Bio::SeqIO;

my $src       =$ARGV[0];
my $fasta     =$ARGV[1];
my @src_name = split(/\./, $src );
pop @src_name ;
my $fileout1=join('.',@src_name).'.hash';
my $fileout2=join('.',@src_name).'.range.hash';
my $fileout3=join('.',@src_name).'.seq.hash';

##########################################################################################
print "Create Gene annotation hash","\n";
open(SRC,"$src")||die "can not open file:";

my %gene2records = ();
my %range2gene = ();

my $total_size = 5000000;
my $overlap    =  100000;


while(<SRC>) 
{
	if(m/^#/)
	{
		next;
	}
	
    my @line = split(/\t/,$_);
        
    if($line[8] =~ m/([^\r\n]+)/){
    	  $line[8] = $1;
    }    
    my $id='';
    my $name='';
    my $parent = '';
    if($line[8] =~m/ID\=([^\;]+)/){
    		$id=$1;
    }
    	
    my @id = split(/\./,$1);
	my $gene_gff = $id[0].'.'. $id[1];
	
	my  $print = 0;	
	if (uc($line[2]) eq 'GENE'){
    	$print = 1;
    	
		    	### get gene range
		    	my $ooo = $id."\t".$line[3]."\t".$line[4];
		    	
		    	## default
		    	my $site_in =$line[3];
		    	my $site_short = int($site_in/$total_size);    	
		    	my $site_check = $line[0].':'.$site_short;
		    	$range2gene{$site_check}{ $ooo} = 1;
		    	## before
		    	$site_short = int(($site_in-$overlap)/$total_size);	
		    	if($site_short < 0){
		    	  	 $site_short = 0 ;
		    	}
		    	$site_check = $line[0].':'.$site_short;  
		    	$range2gene{$site_check}{ $ooo} = 1;
		    	
		    	## after 
		    	$site_in =$line[4];
		    	$site_short = int(($site_in+$overlap)/$total_size);   	    	
		    	$site_check = $line[0].':'.$site_short;  
		    	$range2gene{$site_check}{$ooo} = 1;
		    	
    }
    else{
    	if($id[2] == 1){
    		$print = 1;
    	}
    }
    
    if($print == 1){
    	if(not exists $gene2records{$gene_gff}){
   	  	    my   @buff=();
   	  	    push @buff, join("\t",@line);
   	  	    $gene2records{$gene_gff} = \@buff;
   	    }
   	    else{
   	    	push @{$gene2records{$gene_gff}}, join("\t",@line);
   	    }
    }
}# while
    
 store (\%gene2records, $fileout1) 
		or die "Can't store data\n";   
		
##########################################################################################
print "Create Gene Range hash","\n";
		
### check range hash	
my 	%range2gene_out = ();
for my $range(keys %range2gene){
	 my @out = ();
	 my @hits = keys %{$range2gene{$range}};
	 for (@hits){
	 	 my @ooo = split(/\t/,$_);
	 	 push @out,[@ooo ];
	 }
	 @out = sort {$a->[1]<=>$b->[1]} @out ;
	 $range2gene_out{$range} = \@out;
}
 store (\%range2gene_out, $fileout2) 
		or die "Can't store data\n";   	
		
 ##########################################################################################
print "Create Gene Seq hash","\n";
my %gene2seq = ();
open(SSS,"$src")||die "can not write file";

my @array=();

while (my $line=<SSS>){
     chomp $line;   
     if($line=~ m/^\#/)   {
        	next;
     }  
     my @temp =     split(/\t/,$line);

     if(uc($temp[2]) eq 'GENE'){
          my $id = '';
          if($temp[8]=~ m/Name\=([^\s]+)/){
          	 $id =$1;
          }
           $temp[8] = $id;
           push @array,[@temp];      
     }
}


my $seq_obj = new Bio::SeqIO(-format => 'largefasta',                                   
                                                    -file   => $fasta ); 

while(my $pseq=$seq_obj->next_seq()){  
     my $name=substr($pseq->display_id,0); 
      	print $name,"\n";
   for(my $i=0;$i<@array;$i++){
      #$chr=substr $array[$i][0],6,2;
      my $chr=$array[$i][0];
      
      if($name eq $chr){
         my $id= $array[$i][8];
        
         
         my $dire =$array[$i][6];
         my $site1=$array[$i][3];
         my $site2=$array[$i][4];
         
         my $seq= $pseq->subseq($site1,$site2);  
          	
        
        if($dire eq '-'){        
           $seq=re_co($seq);
        }
        $gene2seq{$id}=$seq;
         
     }
   }
}


sub re_co
{
my $string="";
my $slen="";
my $temp;
my $stemp="";
my $sout="";

  $string=$_[0];
  $slen=length($string);

  for ($temp=0; $temp < $slen; $temp++) 
  {
    $stemp=substr ($string,$temp,1);
    $stemp= &com($stemp);
    $sout=$stemp.$sout;     
  }

return $sout;
1;
}
sub com
{
  my $in=$_[0];
  my $out;
  if    ($in eq "A"){$out="T";}
  elsif($in eq "T"){$out="A";}
  elsif($in eq "G"){$out="C";}
  elsif($in eq "C"){$out="G";}  
  else {$out='N';}
return $out;
1;  
}
    
 store (\%gene2seq, $fileout3) 
		or die "Can't store data\n"; 
		
		
		
#my $arrRef = retrieve('/usr/local/genome/galaxy_dist/tools/localTools/gModInfo.perlData');
#my $arrRef = retrieve('gModInfo.perlData');