#!/bin/perl -w

use strict;
use Class::Struct;
use Storable;
use Bio::DB::Sam;
use Vcf;
my $input_tag  = $ARGV[0]; 
my $gene_input = $ARGV[1]; 
my $genome_version   = $ARGV[2];
my $random  = $ARGV[3];

my $batmis_dir = '';
#my $batmis_dir = '/panfs/pstor.storage/grphomes/cjtlab/lxue/Batmis/bin/';

if(not defined $random){
	 $random = int(rand 10000000);
}

#########################################################################
### get genomic sequences
$input_tag  = "gene";

#my $random = int(rand 10000000);
my $g_all            =  $gene_input.'_'."AllGrna_".$random.'.fa';
my $gff_out          =  $gene_input.'_'."GeneFeature_".$random.'.gff3';
my $mapping_out      =  $gene_input.'_'."Mapping_".$random.'.sam';
my $mapping_out_bam  =  $gene_input.'_'."Mapping_".$random.'.bam';
my $mapping_sort_pre =  $gene_input.'_'."Mapping_".$random.'_sort';
my $mapping_sort_out =  $gene_input.'_'."Mapping_".$random.'_sort.bam';
my $candidate_out = $gene_input.'_'."Candidate_".$random.'.gff3';
my $miss_out = $gene_input.'_'."MissTarget_".$random.'.txt';

my $gff_file   =    $genome_version.'.gene.hash';
my $range_file =   $genome_version.'.gene.range.hash';
my $gene_seq_file  =   $genome_version.'.gene.seq.hash';	
my $genome_vcf_file  =   $genome_version.'.vcf.gz';	

my $arrRef = retrieve("$gff_file");
my %hash_gff = %{$arrRef};

my $arrRef2 = retrieve("$range_file");
my %hash_range = %{$arrRef2};


		
#################################################################################
### input processing
## checktime
 my ($sec,$min,$hour) = localtime(time);
 my @ttt = ($sec,$min,$hour);
 print '----------------------------------\n';
 print join("\t",@ttt  )," start\n";



print "Check input seq\n";
my $seq_in  = '';
my $len_input = length($gene_input);

print $len_input," length input\n";
my @gene_site = ();

if($input_tag  eq "gene"){
	
		###################################################
		## get the GFF records

		open(GFFOUT,">$gff_out")||die "can not write gff file";  
	
		
		if(exists $hash_gff{$gene_input}){
			print GFFOUT join("\n", @{$hash_gff{$gene_input}}),"\n";
			
			for ( @{$hash_gff{$gene_input}}){
				my @ttt = split(/\t/,$_);				
				if(uc($ttt[2]) eq 'GENE'){
					@gene_site =@ttt ;
				}
			}
		}
		else{
			print "this is not one gene\n";
			die;			
		}
		
		
	   ###############################################
       #### get the sequences
       my $arrRef3 = retrieve("$gene_seq_file");
       my %gene2seq = %{$arrRef3};
       if(exists $gene2seq{$gene_input}){
       	  $seq_in  = $gene2seq{$gene_input};
       }
       %gene2seq = ();
}## gene
else{
	$seq_in  = $gene_input;
}

#######################################################################
### get all grna
print "Get all gRNA\n";
open(TGT,">$g_all")||die "can not write file";
my %grna2seq = ();

my $full = $seq_in ;
$full =~ s/\s//g;
$full = uc($full);

my $len = length($full);

my @start_gg = ();
while($full =~ m/(?=GG)/g){
	push @start_gg,$-[0];
}

my @start_cc = ();
while($full =~ m/(?=CC)/g){
	push @start_cc,$-[0];
}

for (@start_gg){
	my $start = $_-21;
	if($start>=0){
		my $seq = substr ($full,$start,23);
		print TGT ">$gene_input".':'."gg",$_,"\n",$seq,"\n";
		$grna2seq{$gene_input.':'."gg".$_} = $seq;		
	}
}

for (@start_cc){
	my $start = $_;
	if($start<=$len-23){
		my $seq = substr ($full,$start,23);
		$seq = rev_com($seq );
		print TGT ">$gene_input".':'."cc",$_,"\n",$seq,"\n";
		$grna2seq{$gene_input.':'."cc".$_} = $seq;			
	}
}
close(TGT);


#####################################################################
## get orignal site of gRNA
my %grna2originalSite = ();
for my $grna ( keys %grna2seq){
	   my @out   =  @gene_site;
	   my @new_site = &get_new_site($grna,$out[3],$out[4],$out[6]);
	   my $new_start = $new_site[0];
	   my $new_direct= $new_site[1];
	   my $perfect =  $out[0].":".$new_start.$new_direct;
	   $grna2originalSite{$grna}{ $perfect} = 1;
}

sub get_new_site{
	my ($grna,$left,$right,$dire) = @_;
	
	my $relative_num = 0;
	my $out = 0;
	my $dire_out = '+';
	if($grna=~m/gg(\d+)/){
		$relative_num = $1;
		if($dire eq '+'){
	         $out  = $left + $relative_num - 21;
	         $dire_out = '+';
	    }
	    else{
	     	 $out  = $right - ($relative_num + 1);
	     	  $dire_out = '-';
	    }
	}
	elsif($grna=~m/cc(\d+)/){
		$relative_num = $1;
		if($dire eq '+'){
	         $out  = $left + $relative_num + 0;
	         $dire_out = '-';
	    }
	    else{
	     	 $out  = $right - ($relative_num + 22);
	     	 $dire_out = '+';
	    }
	}
	else{
		die "Wrong input\n";
	}
	return ($out,$dire_out);
####  +GG  left  + (gg-21)
####  -GG  right - (gg + 1)
####  +CC  left  + (cc+ 0)
####  -CC  right - (cc+22)
}


###################################################################################################
### gRNA mapping 
### we have two versions here


####################################################################################################
my $genome_db =  $genome_version.'.fa';

my ($sec_1,$min_1,$hour_1,$mday_1) = localtime(time);
my $min_start = $min_1+$hour_1*60+$mday_1*24*60;
my $command = $batmis_dir. 'batmap -q '.$g_all.' -g '.$genome_db.' -n6 -mall  -o '.$mapping_out;
#batmap -q all_grna_4803360.fa -g Pta717s_v1.1.fa -n6 -o query_result.txt

my $system_status = system($command) ;
my ($sec_2,$min_2,$hour_2,$mday_2) = localtime(time);
my $min_end = $min_2+$hour_2*60+$mday_2*24*60;


my %grna2record = ();   

if($system_status != 0 or $min_end-$min_start >20 ){
	### too much memory may be required go to slow version
	
	## split the files
	
	&run_sam_directly($g_all);
	
}
else{
	## directory version
	%grna2record = ();   
	# covert files
	$command = 'samtools view -bS '.$mapping_out. ' >'. $mapping_out_bam;
    my $status1 = system($command);

    $command = 'samtools sort  '.$mapping_out_bam. '  '.$mapping_sort_pre ;
    my $status2 = system($command);

    $command = 'samtools index '.$mapping_sort_out ;
    my $status3 = system($command);
    
    
    ($sec,$min,$hour) = localtime(time);
	 @ttt = ($sec,$min,$hour);
	 print '----------------------------------\n';
	 print join("\t",@ttt  )," mapp done\n";
	 
    
    
    ### gRNA  filter candidate
    if($status1 != 0 or $status2 != 0  or $status3 != 0 ){
        	 &run_sam_directly($g_all);
        	
    }
    else{
    	     &filter_bam($mapping_sort_out);
    }


     
	
}


###################################################################################################
sub run_sam_directly{
	my $g_all_in = shift @_;
	%grna2record = ();   
	### here split gRNA files
  open (GRNAIN,"<$g_all_in") || die "Cannot open gRNA file\n";
  my $g_num = 0;
  while(my $line = <GRNAIN>){	
	if($line=~ m/^\>/){
		$g_num++;
		my $grna_each = 'p'.$g_num.'_'.$g_all;		
		open (GRNAOUT,">$grna_each") || die "Cannot write each gRNA\n";		
		print GRNAOUT $line;
		$line = <GRNAIN>;
		print GRNAOUT $line;		
		close(GRNAOUT);
		my $mapping_out_each = 'p'.$g_num.'_'.$mapping_out;

		### mapping each gRNA
		my $command = $batmis_dir. 'batmap -q '.$grna_each .' -g '.$genome_db.' -n6 -mall  -o '.$mapping_out_each;
        system($command);
        
        #batmap -q all_grna_4803360.fa -g Pta717s_v1.1.fa -n6 -o query_result.txt

      #############################################################################################
		### gRNA  filter candidate
		
		 ($sec,$min,$hour) = localtime(time);
		 @ttt = ($sec,$min,$hour);
		 print '----------------------------------\n';
		 print join("\t",@ttt  )," mapp done\n";
		 
		 ## read sam directly
		 &filter_sam_directly($mapping_out_each);
		 ## the data is load into hash %grna2record;
		 
		 $command = 'rm '.$mapping_out_each;
         system($command);
         $command = 'rm '.$grna_each;
         system($command);
	}
  }
}


sub filter_sam_directly{
	my $sam_in = shift @_;
	#### only check first 1000 records
    my $limit_num = 500; 
    open (SAM,"$sam_in")  || die "can not open sam file  $sam_in\n";
    my $count = 0;
    
    while(my $line_in = <SAM>){
	     if ($line_in=~ m/^\@/){
		     next ;
	     }
	     $count ++;
	  
	     if($count > $limit_num){
	 	      last;
	     }
		      	     
		      	  my @lines = split(/\t/,$line_in);
		      	  my ($id, $flag,$seqid,$start,$score,$cigar)  = @lines;
		      	  
		      	  my $query_dna = $lines[9];
		      	     
		      	  for(1..10){
		      	     shift  @lines;
		      	  }
		      	  ### tags
		      	  my $md_tag = '';
		      	  for (@lines){
		      	  	  if(m/MD\:Z\:(\w+)/){
		      	  	  	  $md_tag = $1;
		      	  	  }
		      	  }
		      	  my $nm_tag = 0;
		      	  
		      	  for (@lines){
		      	  	  if(m/NM\:i\:(\w+)/){
		      	  	  	  $nm_tag = $1;
		      	  	  }
		      	  }
		      	  
		      	  
		      	  my $strand = "+";
     
				  if($flag & 16){
				      	  $strand = "-";
				  }
     
			  my $ref_dna = &get_refrence_from_sam ($query_dna,$cigar,$md_tag);
			     
			  if ( $strand eq "-"){
			     	  $ref_dna    = &rev_com($ref_dna);
			     	  $query_dna  = &rev_com($query_dna);
			  } 
			     

			 my $site_all = ''; #empty currently
	 	     my @out = ($id,$seqid,$start,$strand,$cigar,$query_dna,$ref_dna,$md_tag,$site_all,$nm_tag);
                       
             if(not exists $grna2record{$id}){
                  my @buff = ();
                  push @buff,[@out];
                  $grna2record{$id} =\@buff;
                  #print $id,"\t Into the hash\n";
             }
             else{
                  push @{$grna2record{$id}},[@out];
             }
	  
          }# read sam
}




sub filter_bam {
	my $bam_in = shift @_;
	## read bam
    my $sam = Bio::DB::Sam->new(-bam  =>"$bam_in");
    my @targets    = $sam->seq_ids;
    foreach my $target (@targets){
	    my @alignments = $sam->get_features_by_location( -seq_id => $target);
	    for my $a (@alignments) {
	 	      my $id  = $a->name;
	 	     #alignment information in the reference sequence
		      my $seqid  = $a->seq_id;
		      my $start  = $a->start;
		      #my $end    = $a->end;
		      #strand is stored as -1 or 1
		      my $strand = $a->strand;
		      $strand = '+' if $strand =~ /^1$/;
		      $strand = '-' if $strand =~ /^-1$/;
		      my $cigar  = $a->cigar_str;
		      my $md_tag = $a->get_tag_values('MD');
		      my $nm_tag = $a->get_tag_values('NM');
		      
              my $ref_dna   = $a->dna;        # reference sequence bases
              my $query_dna = $a->query->dna; # query sequence bases
              
              if($strand eq '-'){
	 	      	  $query_dna  = rev_com($query_dna);
	 	      	  $ref_dna    = rev_com($ref_dna);
	 	      }
	 	      my $site_all = ''; #empty currently
	 	     my @out = ($id,$seqid,$start,$strand,$cigar,$query_dna,$ref_dna,$md_tag,$site_all,$nm_tag);
                       
             if(not exists $grna2record{$id}){
                  my @buff = ();
                  push @buff,[@out];
                  $grna2record{$id} =\@buff;
                  #print $id,"\t Into the hash\n";
             }
             else{
                  push @{$grna2record{$id}},[@out];
             }
            
	    }# alignment
     }# target

}# filter bam



##############################################################################
##### load hash tables

my $seqid_vcf = $gene_site[0];
my $start_vcf = $gene_site[3];
my $end_vcf = $gene_site[4];
my $strand_vcf = $gene_site[6];
    
my $addr_out = &build_snp_hash($seqid_vcf,$start_vcf,$end_vcf,$strand_vcf);
my %hash_vcf = %{$addr_out};
    


######################################################################################
### generate mismatch table and score

 ($sec,$min,$hour) = localtime(time);
 @ttt = ($sec,$min,$hour);
 print '----------------------------------\n';
 print join("\t",@ttt  )," read sam  done\n";
 
 
 
open(MISS,">$miss_out")||die "can not write miss file";   
my @title = qw/Candidate	Sequence  Hit	Score	MMs	Locus	Gene	Region/;
print MISS join("\t",@title ),"\n";
#### generate candiate file
open(CAND,">$candidate_out")||die "can not white file"; 
@title = qw/	Chromosome TargetSeq  Grade LeftSite RightSite SNP717 Direction Score gRNAname gRNAseq  Gene Region /;
print CAND '#'.join("\t",@title ),"\n";

my @candidate_all = ();
my @miss_all      = ();

for my $grna (sort keys %grna2record){
	
	#print $grna,"\n";
	## each gRNA
	my @hits = @{$grna2record{$grna}};

	my @all_scores = ();
	my @best =();
	for my $line_in (@hits){
		  my ($id,$seqid,$start,$strand,$cigar,$query_dna,$ref_dna,$md_tag,$site_all,$nm_tag)= @{$line_in};
		  
		  my $perfect =  $seqid.":".$start.$strand;
		  if(exists $grna2originalSite{$id}{$perfect}){
		  	  @best = @{$line_in};
		  }
		  else{ # not original
				  if(uc($ref_dna) =~ /GG$/ or uc($ref_dna) =~ /AG$/ or uc($ref_dna) =~ /GA$/){
				  	    my @mis_site = &seq2mismatch($query_dna,$ref_dna);
				  	    if(@mis_site+0 <= 4){
			 	      	    	my $score = &sites2score(@mis_site);
			 	      	    	push @all_scores ,$score;
			 	      	    	### out mismatch 
			 	      	    	my $mis_site_num = @mis_site + 0;
			 	      	    	my $cut_site = $start + 19;
							    if($strand  eq '-'){
									 $cut_site = $start + 3;
							    }
							    my @anno = &get_anno($seqid, $cut_site);
			                    my $locus = $seqid.':'. $cut_site.$strand;
			                    my $tag_mis = '0M';
			                    if($mis_site_num>0){
			                    	 my @ttt = ();
			                    	 for(@mis_site){
			                    	 	 push @ttt,($_+1);
			                    	 }
			                    	 $tag_mis = $mis_site_num.'M('.join(':',@ttt).')'; 
			                    }
			                    ## out mis hits
			                    my @out = ($id,$query_dna,$ref_dna,$score  ,$tag_mis,$locus,$anno[0],$anno[1]);
			                   # print MISS join("\t",@out),"\n";
			                    push @miss_all,[ @out];
				  	    } # less four 
				  }# match AG GG
		  }
    } ## for mismatch hits 
    
       my $sum_score = 0;
	   for(@all_scores ){
				$sum_score = $sum_score +$_;
		}
	   my $grna_score = 100/(100+$sum_score);
	   my $grade = "g";
	   if($grna_score < 0.5){
		   $grade = 'r';
	   }	   

	     	### output candidate into array
	        my ($id,$seqid,$start,$strand,$cigar,$query_dna,$ref_dna,$md_tag,$site_all,$nm_tag) = @best;

			my $cut_site = $start + 19;
			if($strand  eq '-'){
					$cut_site = $start + 3;
			}
            my $snp = '';
 	        my @anno = &get_anno($seqid, $cut_site);
	 
	        my @out = ($seqid,$ref_dna,$grade,$start,($start+22),$snp,$strand, $grna_score,$id,$query_dna,$anno[0],$anno[1]);
	        # print CAND join("\t",@out),"\n";
	        push @candidate_all,[@out];
}# grna
	   
	   

###############################################################################
### write candiate
@candidate_all = sort{$b->[7]<=>$a->[7]}@candidate_all;
my %id2newid = ();
my $num_counting = 0;
for (@candidate_all){
	my @out = @{$_};
	
	my ($seqid,$ref_dna,$grade,$start,$end,$snp,$strand)= @out;
	
	#print join("\t",($ref_dna,$seqid,$start,$strand)),"\n";
	#$snp = 'N';
	$snp = &get_snp_from_hash($ref_dna,$seqid,$start,$strand);
	
    $out[5] = $snp;
    
	$num_counting++;
	my $id = $out[8];
	
	my $new = 'Guide-'.$num_counting;
	$id2newid{$id}=$new;
	
	$out[8]='ID='.$new ;
	print CAND join("\t",@out),"\n";
}
### write mis target
for (@miss_all){
	my @out = @{$_};	
	my $id = $out[0];
	my $new = $id2newid{$id};
    $out[0] = $new;

	print MISS join("\t",@out),"\n";
}
## writing
###############################################################################
 ($sec,$min,$hour) = localtime(time);
 @ttt = ($sec,$min,$hour);
 print '----------------------------------\n';
 print join("\t",@ttt  )," all  done\n";
 
 
    
sub sites2score{
    	my @mis_site = @_; # base 0
        my @weight_all = (0,0,0.014,0,0,0.395,0.317,0,0.389,0.079,0.445,0.508,0.613,0.851,0.732,0.828,0.615,0.804,0.685,0.583);

	          my $mis_site_num = @mis_site + 0;
	          ## score 3
	          my $score_p3 = 1;
	          if($mis_site_num>=1){
	          	  $score_p3 = 1/($mis_site_num*$mis_site_num);
	          }
	          ## score 2
	           my $score_p2 = 1;	          
	           if($mis_site_num>=2){
	           	   # others equal 1
	           	   my @distance = ();
	          	   for(my $i=0;$i<($mis_site_num-1);$i++){
	          	   	   my $dis = $mis_site[$i+1]-$mis_site[$i];
	          	   	   push @distance,$dis;
	          	   }
	          	   my $sum_dis = 0;
				   for( @distance){
					    $sum_dis = $sum_dis + $_;
				   }
				   my $mean_dis = $sum_dis/(@distance+0);
			       $score_p2 = 1/((((19-$mean_dis)/19)*4)+1);
	           }
	           
	           ## score 1
	           my $score_p1 = 1;
			   ## get score part 1
			   for(@mis_site){		
						$score_p1 = $score_p1*(1-$weight_all[$_]);
			   }
	           	           
	           my $score_hit = $score_p1*$score_p2 *$score_p3*100;
	           return $score_hit;
}# get score
    
    
    


sub seq2mismatch{
	my ($query_dna,$ref_dna) = @_;
	my @out = ();
	# end with letter
	# end with number
	
	if(length($query_dna) != length($ref_dna)){
		die "not same length\n";
	}
	
	for (my $i=0;$i<length($query_dna);$i++){
		if((substr $query_dna,$i,1) ne (substr $ref_dna,$i,1) ){
			if($i<20){
				push @out,$i;
			}			
		}
	}
	
	return @out;
}





###################################################################################################


#######################################
### read sam file

sub get_refrence_from_sam {
	my ($read,$cigar,$mdz) = @_;
	
	## read cigar infor
	my $start_read  = 0;
	my $seq_out = '';
	
	while ($cigar =~ /(\d+)([MIDNSHP=X])/g) {
			my $count = $1;
			my $identifier = $2;	
			if($identifier eq 'M'){
				$seq_out = $seq_out.(substr $read,$start_read, $count);
			    $start_read = $start_read+$count;
			} 
			elsif($identifier eq 'I'){
				$seq_out = $seq_out.&get_dash($count);
				 $start_read = $start_read+$count;
			}
			elsif($identifier eq 'D'){
				  $seq_out = $seq_out.&get_underline($count);
			}
			else{
				die "new letter in cigar\n";
			}
			
	}
	
	$seq_out =~ s/\-//g;
	## read md z infor
	
	$start_read  = 0;
	if ($mdz !~ m/^(\d+)/) {
		$mdz = '0'.$mdz;
	}
	
	my $ref_seq = '';
	while ($mdz  =~ /(\d+)([\^ATGCN]+)/g) {
		    my $count = $1;
			my $identifier = $2;			
			
			$ref_seq  = $ref_seq . (substr $seq_out, $start_read, $count );
			$start_read = $start_read+$count;
			
			if($identifier =~ m/^\^/){
				# deletion
				my $deletion = substr $identifier,1;				
				my $deltion_len = length($deletion);
				$ref_seq  = $ref_seq . $deletion ;
				$start_read = $start_read+$deltion_len;
				
			}
			else{
				# mis matches
				$ref_seq  = $ref_seq . $identifier ;
				$start_read ++;
			}
	}
	if($mdz  =~ m/(\d+)$/){
		my $count = $1;
		$ref_seq  = $ref_seq . (substr $seq_out, $start_read, $count );
	}
	return $ref_seq ;

}

sub get_dash{
		my $num = $_[0];
		my $out = '';
		if( $num >=1){
			for (1..$num){
				$out = $out.'-';
			}
		}
		return $out;
}

sub get_underline{
		my $num = $_[0];
		my $out = '';
		if( $num >=1){
			for (1..$num){
				$out = $out.'_';
			}
		}
		return $out;
}


sub get_anno{
	my ($scaffold, $cut_site) = @_;
	my $total_size = 5000000;
	my @out = ("error","top level");
	my $site_check = $scaffold.':'.int( $cut_site/$total_size);
    
    if(exists $hash_range{$site_check}){
    	 my @genes = @{$hash_range{$site_check}};
    	 if($cut_site>=$genes[0][1] and $cut_site<=$genes[-1][2] ){
              @out = &check_loop($cut_site,\@genes);
    	 }
    	 elsif($cut_site<$genes[0][1]){
    	 	 @out=("next to ".$genes[0][0],"intergenic");
    	 }
    	 elsif($cut_site>$genes[-1][2]){
    	 	 @out=("next to ".$genes[-1][0],"intergenic");
    	 }
    	 else{
    	 	  @out=("error ","unknown");
    	 }
    }
    else{
    	 @out=("far from gene","intergenic");
    }
	return @out;
}

	 	
sub check_loop{
	my ($cut_site,$arratAdrr) = @_;
	my @gene_in = @{$arratAdrr};
	my @out =("error","gene level");
	for(my $i=0;$i<(@gene_in-1);$i++){
		if($cut_site >= $gene_in[$i][1] and $cut_site <= $gene_in[$i][2]){
			my $gene = $gene_in[$i][0];
			@out = &get_exon($cut_site,$gene);
			last;
		}
		elsif($cut_site > $gene_in[$i][2] and $cut_site <$gene_in[$i+1][1] ){
			@out =($gene_in[$i][0].'-'.$gene_in[$i+1][0],"intergenic");
			last;
		}
	}
	## last gene;
	if($cut_site >= $gene_in[-1][1] and $cut_site <= $gene_in[-1][2]){
		 my $gene = $gene_in[-1][0];
		 @out = &get_exon($cut_site,$gene);
	}
	return @out;
}
    	 	
sub get_exon{
	my ($cut_site,$gene)=@_;
	my @out =("error","exon level");
	
	if(exists $hash_gff{$gene}){
		my @exon = ();
	    for ( @{$hash_gff{$gene}}){
				my @ttt = split(/\t/,$_);				
				if(uc($ttt[2]) ne 'GENE' and uc($ttt[2]) ne 'MRNA'){
					push @exon,[@ttt];
				}
		}
		### check exon
		@exon = sort {$a->[3]<=>$b->[3]}@exon;
		
		if($cut_site <$exon[0][3]){
			## left
			if($exon[0][6 eq '+']){
				 @out =($gene,"five_prime_UTR");
			}
			else{
				@out =($gene,"three_prime_UTR");
			}
		}
		elsif($cut_site > $exon[-1][4]){
			# right 
			if($exon[0][6 eq '+']){
				 @out =($gene,"three_prime_UTR");
			}
			else{
				@out =($gene,"five_prime_UTR");
			}
		}

		for (my $i=0;$i<(@exon-1);$i++){
			 if($cut_site>=$exon[$i][3] and $cut_site<=$exon[$i][4]){
			 	  @out =($gene,$exon[$i][2]);	
			 	  last;
			 }
			 elsif($cut_site> $exon[$i][4] and $cut_site<$exon[$i+1][3]){
			 	  @out =($gene,"intron");
			 	  last;
			 }
		}
		
		if($cut_site>=$exon[-1][3] and $cut_site<=$exon[-1][4]){
			 	  @out =($gene,$exon[-1][2]);
	    }
	}
	else{
		@out =($gene,"unknown");			
	}	
	return @out;	
}
 

#############################################################
### delete files

 
my $command2 = 'rm '.$mapping_out ;
system($command2);
 $command2 = 'rm '.$mapping_out_bam;
system($command2);	 
 $command2 = 'rm '. $mapping_sort_out;
system($command2);	 
 $command2 = 'rm '. $mapping_sort_out.'.bai';
system($command2);

 $command2 = 'rm '.$g_all;
system($command2);		 
	        
sub rev_com {
   my ($seq) = @_;
   $seq = reverse($seq);
   $seq =~ tr/ACGT/TGCA/;
   return($seq);
}
 
#########################################
sub get_snp_from_hash{
    my ($ref_dna,$seqid,$start,$strand) = @_;
  
    if($strand eq '-'){
    	$ref_dna = rev_com($ref_dna);
    }
    my @ref_all = split(//,$ref_dna);
    my @out_all = ();
    
    my $snp_num = 0;
    my @snp_site = ();
    my @snp_type = ();
    
    
    
    for(my $i=0; $i<@ref_all;$i++){
    		my $site_new = $start+$i;
    		my $ref_new  = $ref_all[$i];
    		my $index_new = $seqid.':'.$site_new.':'.$ref_new;
    		my $out = $ref_new;
    		if(exists $hash_vcf{$index_new}){
    			$snp_num++;    			
    			$out = $hash_vcf{$index_new};
    			push @snp_site,($i+1);
    			my $type = 'S';
    			if($out eq '-'){
    				$type = 'D';
    			}
    			elsif(length($out)>1){
    				$type = 'I';
    			}
    			push @snp_type ,$type;
    		}
    		push @out_all, $out;
    }
    my $out_string = join('',@out_all);
    
    if($strand eq '-'){
    	$out_string = rev_com_snp($out_string);
    	## site
    	my @snp_site_new = ();    	
    	for(@snp_site){
    		unshift @snp_site_new,(24-$_);    		
    	}
    	@snp_site =  @snp_site_new;
    	## type
    	my @snp_type_new = (); 
    	for(@snp_type) {
    		unshift @snp_type_new,$_;    
    	}
    	@snp_type = @snp_type_new;
    	
    }
    
    if($snp_num >0){    	
    	my @string_snp = ();
    	for(@snp_site){
    		my $t_in = shift @snp_type;
    		push @string_snp,$_.$t_in;
    	}
    	$out_string = $out_string.' '.$snp_num.' V('.join(':',@string_snp).')';
    }
    
    return $out_string;
    
}	


        
sub rev_com_snp {
   my ($seq) = @_;
   $seq = reverse($seq);
   
   $seq =~ tr/ACGTMKRYBDHV/TGCAKMYRVHDB/;
   
my @trans = qw/
A A T
T T A
G G C
C C G

AT  W W
CG 	S S

AC  M K
GT  K M
AG  R Y
CT  Y R
CGT B V
AGT	D 	H
ACT H 	D
ACG V 	B

ACGT N  N
/;
      return($seq);
}  


sub build_snp_hash{
	my ($seqid,$start,$end,$strand) = @_;

	my $vcf = Vcf->new(file=>$genome_vcf_file,region=>'Chr01:1000-2002');

     $vcf->parse_header();
     
     ## set regions 
     my $start_new = $start - 2000;
     if( $start_new < 1){
     	 $start_new = 1;
     }
     my $end_new   =$end + 2000;

    # print $seqid,"\n";
     
     my $contig = $vcf->get_header_line(key=>'contig',ID=>$seqid);
     if($contig){
     	   my %hash_in = %{$$contig[0]};
           my $total_length = $hash_in{"length"};

		   if( $end_new > $total_length){
		     	 $end_new = $total_length;
		   }
     }

     
     my $new_region = $seqid.':'.$start_new.'-'.$end_new;
     
     $vcf = Vcf->new(file=>$genome_vcf_file,region=>$new_region);
     $vcf->parse_header();
     my %vcf_hash = ();
     ### read SNP at region
     while (my $x=$vcf->next_data_array()) {
         my ($chr,$pos,$id,$ref,$alt) = ($$x[0],$$x[1],$$x[2],$$x[3],$$x[4]);
         my  $index = $chr.':'.$pos.':'.$ref;
         my  @alt_original = split(/\,/,$alt);
         my  $ref_len = length($ref);
         if($ref_len == 1){
		    	 for my $each (@alt_original){
		    		$vcf_hash{$index}{$each}=1;
		    	 }
         } # ref eq 1
         else{ # ref length >1
	         	 my @ref_all = split(//,$ref);
	         	 if(@ref_all+0!= $ref_len ){
	    		      die "length not equal";
	    	     }
			     for my $each (@alt_original){
			    		
			    		my @alt_all = split(//,$each);
			    		for(my $i=0;$i<@ref_all;$i++){
			    			my $site_new = $pos+$i;
			    			my $ref_new  = $ref_all[$i];
			    			my $alt_out = '-';
			    			
			    			#### last record in ref
			    			if($i==(@ref_all-1)){
			    				if(@alt_all+0>0){
			    					$alt_out = join('',@alt_all);
			    				}
			    			}
			    			else{
			    				if(@alt_all+0>0){
			    				   $alt_out = shift @alt_all;
			    			    }
			    			}
			    			my $index_new = $chr.':'.$site_new.':'.$ref_new;
			    		    $vcf_hash{$index_new}{$alt_out}=1;
			    		}## for index
			     } ## each alt
        } ## ref lenght >1 
     }  ### read while  
     
     ## regnerate the  hash
     
     my %new_hash = %{&regnerate_hash(\%vcf_hash)};
     return \%new_hash ;
}# while


sub regnerate_hash {
	my $hash_addr = $_[0];
	
	my %vcf_hash = %{$hash_addr};
	my %vcf_site = ();
	
	## translate table
	my @trans = qw/
			A A
			T T
			G G
			C C
			AT  W
			CG 	S 	
			AC  M 	
			GT  K 
			AG  R 
			CT  Y 
			CGT B 
			AGT	D 	
			ACT H 	
			ACG V 	
			ACGT N 
			/;
			my %nt2translate =();

			while(@trans){
				my $key = shift @trans;
				my $value = shift @trans;
				$nt2translate{$key}=$value;
			}
			## translate the table
			
		for my $index (sort keys %vcf_hash){	
				my @hit = keys %{$vcf_hash{$index}};
				my $out = $hit[0];
				if(@hit+0>1){ # many hits
				    my @check_single_nt   = ();
				    my @check_multiple_nt = ();
				    for(@hit){
				    	if($_ !~ m/\-/){
				    		if(length($_) ==1){
				    			 push @check_single_nt,$_;
				    		}
				    		else{
				    			 push @check_multiple_nt,$_;
				    		}
				    	}
				    }
				    
				    if(@check_single_nt>0){
				    	  @check_single_nt = sort @check_single_nt;
				          my $check_string = join('',@check_single_nt);
				          my $check_string2 = join('z',@hit);
				          if(exists $nt2translate{$check_string }){
				    	       $out = $nt2translate{$check_string};
				          }
						  else{
						    	die "$check_string No translate 1 $check_string2\n";
						  }
				    }
				    else{
				    	 @check_multiple_nt = sort @check_multiple_nt;
				    	 $out = $check_multiple_nt[0];
				    }
				}
				
				my @original = split(/\:/,$index);
				if($original[-1] ne $out){
					$vcf_site{$index}=$out;
				}
	  }
	return \%vcf_site;
}



   